from abc import ABCMeta
from configparser import ConfigParser
from pathlib import Path
from typing import Any, Tuple, Type

from pydantic_settings import (
    BaseSettings,
    PydanticBaseSettingsSource,
    SettingsConfigDict,
)
from pydantic_settings.sources import (
    ConfigFileSourceMixin,
    InitSettingsSource,
    JsonConfigSettingsSource,
    TomlConfigSettingsSource,
    YamlConfigSettingsSource,
)


class IniConfigSettingsSource(InitSettingsSource, ConfigFileSourceMixin):
    """
    A source class that loads variables from a Ini file
    """

    def __init__(
        self,
        settings_cls: type[BaseSettings],
        ini_file: Path | None = None,
        ini_file_encoding: str | None = None,
    ):
        self.ini_file_path = ini_file or settings_cls.model_config.get("json_file")
        self.ini_file_encoding = (
            ini_file_encoding if ini_file_encoding is not None else settings_cls.model_config.get("env_file_encoding")
        )
        self.ini_data = self._read_files(self.ini_file_path)
        super().__init__(settings_cls, self.ini_data)

    def _read_file(self, file_path: Path) -> dict[str, Any]:
        with open(file_path, encoding=self.ini_file_encoding) as ini_file:
            file_content = ini_file.read()
            parser = ConfigParser()
            parser.read_string(file_content)
            return getattr(parser, "_sections", {}).get("settings", {})


class SettingsBase(BaseSettings, metaclass=ABCMeta):
    """
    项目设置的基类
    """

    model_config = SettingsConfigDict(
        extra='allow',
        env_file_encoding="utf-8",
        env_nested_delimiter="__",
        env_file=(Path.cwd() / "configs").absolute() / ".env",
    )

    @classmethod
    def settings_customise_sources(
        cls,
        settings_cls: Type[BaseSettings],
        init_settings: PydanticBaseSettingsSource,
        env_settings: PydanticBaseSettingsSource,
        dotenv_settings: PydanticBaseSettingsSource,
        file_secret_settings: PydanticBaseSettingsSource,
    ) -> Tuple[PydanticBaseSettingsSource, ...]:
        """
        自定义配置来源
        Args:
            settings_cls:
            init_settings: 初始化设置
            env_settings:环境变量设置
            dotenv_settings:
            file_secret_settings:加密文件设置

        Returns:

        """
        # 默认的设置
        default_settings = {
            init_settings,
            env_settings,
            dotenv_settings,
            file_secret_settings,
        }
        root_dir = (Path.cwd() / "configs").absolute()

        # json 配置文件
        json_file = root_dir / "settings.json"
        if json_file.exists():
            json_settings_source = JsonConfigSettingsSource(settings_cls, json_file)
            default_settings.add(json_settings_source)

        # ini配置文件
        ini_file = root_dir / "settings.ini"
        if ini_file.exists():
            ini_settings_source = IniConfigSettingsSource(settings_cls, ini_file)
            default_settings.add(ini_settings_source)

        # yaml配置文件
        yaml_file = root_dir / "settings.yaml"
        if yaml_file.exists():
            yaml_settings_source = YamlConfigSettingsSource(settings_cls, yaml_file)
            default_settings.add(yaml_settings_source)

        # toml配置文件
        toml_file = root_dir / "settings.toml"
        if toml_file.exists():
            toml_settings_source = TomlConfigSettingsSource(settings_cls, toml_file)
            default_settings.add(toml_settings_source)

        return tuple(default_settings)



from furl import furl

from hssp.models.net import RequestModel
from hssp.network.response.selector import Selector


class Response:
    headers: dict
    content: bytes
    text: str
    json: dict

    def __init__(
            self,
            url: str,
            status_code: int,
            headers: dict,
            cookies: dict,
            client_cookies: dict,
            content: bytes,
            text: str,
            json: dict,
            request_data: RequestModel,
    ):
        self.request_data = request_data
        self.status_code = status_code
        self.cookies = cookies
        self.client_cookies = client_cookies
        self.headers = headers
        self.text = text
        self.content = content
        self.json = json

        self.url: str = url
        self.domain = furl(self.url).origin
        self.host = furl(self.url).host

        self.selector = Selector(self.text)

    def xpath(self, query: str):
        return self.selector.xpath(query, domain=self.domain)

    def css(self, query: str):
        return self.selector.css(query, domain=self.domain)

    def re(self, regex: str, replace_entities=True):
        return self.selector.re(regex, replace_entities=replace_entities)

    def re_first(self, regex: str, default=None, replace_entities=True):
        return self.selector.re_first(regex, default=default, replace_entities=replace_entities)



import sys
import asyncio
from json import JSONDecodeError

from curl_cffi.requests import AsyncSession
from curl_cffi.const import CurlHttpVersion

from hssp.exception.exception import RequestStateException
from hssp.models.net import RequestModel, ProxyModel
from hssp.network.downloader import DownloaderBase
from hssp.network.response import Response


class CurlCffiDownloader(DownloaderBase):

    def __init__(self, sem: asyncio.Semaphore, headers: dict = None, cookies=None):
        super().__init__(sem, headers, cookies)

        self.client = AsyncSession(
            verify=False,
            headers=self._default_headers,
            cookies=self._default_cookies,
            impersonate="chrome110",
            http_version=CurlHttpVersion.V2_0
        )

    async def close(self):
        await self.client.close()

    @property
    def cookies(self):
        return self.client.cookies.get_dict()

    def set_proxy(self, proxy: ProxyModel | str):
        ...

    async def _download(self, request_data: RequestModel) -> Response:
        if isinstance(request_data.proxy, ProxyModel):
            proxy = request_data.proxy.https or request_data.proxy.http
        else:
            proxy = request_data.proxy
        proxies = {"https": proxy, "http": proxy}

        response = await self.client.request(
            method=request_data.method,
            url=request_data.url,
            params=request_data.url_params,
            data=request_data.form_data,
            json=request_data.json_data,
            cookies=request_data.cookies,
            headers=request_data.headers,
            proxies=proxies,
            timeout=request_data.timeout
        )

        if not response.ok:
            raise RequestStateException(code=response.status_code)

        resp_headers = dict(response.headers)
        resp_content = response.content
        resp_text = response.text
        resp_cookies = response.cookies.get_dict()
        try:
            resp_json = response.json()
        except (JSONDecodeError, UnicodeDecodeError):
            resp_json = None

        return Response(
            url=response.url,
            status_code=response.status_code,
            headers=resp_headers,
            cookies=resp_cookies,
            client_cookies=self.cookies,
            content=resp_content,
            text=resp_text,
            json=resp_json,
            request_data=request_data
        )



import random
import string


def rand_str(length: int, style: str | None = None):
    """
    获取固定长度的随机字符串
    Args:
        style: 格式：默认为大写字母+小写字母+数字
        length: 长度

    Returns:

    """
    if not style:
        style = string.digits + string.ascii_letters
    max_length = len(style)
    if length <= max_length:
        return "".join(random.sample(style, length))

    text = ""
    count = length // max_length
    remainder = length % max_length
    for _ in range(count):
        text += rand_str(max_length)
    if remainder > 0:
        text += rand_str(remainder)
    return text


def random_1():
    """
    返回0-1之间的随机数
    Returns:

    """
    return random.random()



from hashlib import md5, sha256
from Cryptodome.Cipher import AES
from Cryptodome.Util.Padding import pad, unpad


def encrypt_aes_256_cbc(data: bytes, key: bytes, iv: bytes) -> bytes:
    """
    aes_256加密 不填充
    Args:
        data: 加密的数据
        key: key
        iv: iv

    Returns:

    """

    cipher = AES.new(key, AES.MODE_CBC, iv)
    ciphertext = cipher.encrypt(data)
    return ciphertext


def decrypt_aes_256_cbc(data: bytes, key: bytes, iv: bytes) -> bytes:
    """
    aes_256解密 不填充
    Args:
        data: 数据
        key: key
        iv: iv

    Returns:

    """
    cipher = AES.new(key, AES.MODE_CBC, iv)
    ciphertext = cipher.decrypt(data)
    return ciphertext


def encrypt_aes_256_cbc_pad7(data: bytes, key: bytes, iv: bytes) -> bytes:
    """
    aes_256加密 pad7填充
    Args:
        data: 加密的数据
        key: key
        iv: iv

    Returns:

    """

    data = pad(data, 16)
    return encrypt_aes_256_cbc(data, key, iv)


def decrypt_aes_256_cbc_pad7(data: bytes, key: bytes, iv: bytes) -> bytes:
    """
    aes_256解密 pad7填充
    Args:
        data: 数据
        key: key
        iv: iv

    Returns:

    """
    data = decrypt_aes_256_cbc(data, key, iv)
    return unpad(data, AES.block_size)


def md5_hash(data: bytes | str, result_type: str = "hex"):
    """
    md5 hash
    Args:
        data: 数据，字节数据或字符串，字符串或使用utf8编码为字节
        result_type: 返回类型，默认为16进制字符串，bytes为字节

    Returns:
        返回经过md5哈希后的值
    """
    data = data if isinstance(data, bytes) else data.encode('utf-8')
    result = md5(data)
    if result_type == "hex":
        return result.hexdigest()
    else:
        return result.digest()


def sha256_hash(data: bytes | str, result_type: str = "hex"):
    """
    sha256 hash
    Args:
        data: 数据，字节数据或字符串，字符串或使用utf8编码为字节
        result_type: 返回类型，默认为16进制字符串，bytes为字节

    Returns:
        返回经过sha256哈希后的值
    """
    data = data if isinstance(data, bytes) else data.encode('utf-8')
    result = sha256(data)
    if result_type == "hex":
        return result.hexdigest()
    else:
        return result.digest()



import smtplib
from email.message import EmailMessage


def main():
    from_addr = "hao-se@outlook.com"
    to_addrs = ["xhrtxh@gmail.com"]
    access_token = ""

    msg = EmailMessage()
    msg.set_content("测试发送")
    msg['Subject'] = '测试发送'
    msg['From'] = from_addr
    msg['To'] = to_addrs[0]

    server = smtplib.SMTP(host="smtp-mail.outlook.com", port=587)
    server.set_debuglevel(1)
    server.starttls()
    server.docmd('AUTH', f'XOAUTH2 {access_token}')

    server.quit()


if __name__ == '__main__':
    main()



class RequestException(Exception):
    """
    请求发生异常
    """

    def __init__(self, exception_type: str, exception_msg: list):
        self.exception_type = exception_type
        self.exception_msg = exception_msg


class RequestStateException(Exception):
    """
    请求状态发生异常
    """

    def __init__(self, code):
        self.code = code



import asyncio

from hssp.logger.log import Logger
from hssp.network import Net
from hssp.network.downloader import CurlCffiDownloader


async def main():
    url = "https://tools.scrapfly.io/api/fp/ja3"
    net = Net(downloader_cls=CurlCffiDownloader)
    resp = await net.get(url)
    logger.info(resp.text)
    await net.close()


if __name__ == '__main__':
    Logger.init_logger()
    logger = Logger.get_logger("test")
    asyncio.run(main())
